package cn.bunny.service.service.impl;

import cn.bunny.common.constant.MessageConstant;
import cn.bunny.common.service.exception.BunnyException;
import cn.bunny.common.service.utils.JwtHelper;
import cn.bunny.entity.system.Login;
import cn.bunny.entity.system.SysUser;
import cn.bunny.entity.system.SysUserinfo;
import cn.bunny.service.mapper.SysUserMapper;
import cn.bunny.service.service.SysUserService;
import cn.bunny.vo.system.LoginVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author bunny
 * @since 2024-04-22
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    /**
     * 登录
     *
     * @param vo 登录条件
     * @return 返回token
     */
    @Override
    public Login login(LoginVo vo) {
        String username = vo.getUsername();
        String password = vo.getPassword();
        // 查询用户信息
        LambdaQueryWrapper<SysUser> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(SysUser::getUsername, username);
        SysUser sysUser = getOne(wrapper);

        // 用户是否存在
        if (sysUser == null) {
            throw new BunnyException(MessageConstant.ACCOUNT_NOT_FOUND);
        }

        // 判断是否被禁用
        if (sysUser.getStatus() == 0) {
            throw new BunnyException(MessageConstant.ACCOUNT_LOCKED);
        }

        // 判断密码
        String md5DigestAsHexPassword = DigestUtils.md5DigestAsHex(password.getBytes());
        if (!md5DigestAsHexPassword.equals(sysUser.getPassword())) {
            throw new BunnyException(MessageConstant.PASSWORD_ERROR);
        }
        // 添加token
        String token = JwtHelper.createToken(sysUser.getId(), sysUser.getUsername());
        return Login.builder().token(token).build();
    }

    /**
     * 获取用户信息
     *
     * @param request 请求头
     * @return 用户信息
     */
    @Override
    public SysUserinfo getUserinfo(HttpServletRequest request) {
        redisTemplate.opsForValue().set("test", "测试");
        return null;
    }

    /**
     * 根据用户名查询用户信息
     *
     * @param username 用户名
     * @return 用户信息
     */
    @Override
    public SysUser getByUsername(String username) {
        return getOne(Wrappers.<SysUser>lambdaQuery().eq(SysUser::getUsername, username));
    }
}
